import java.util.Random;

/**
 * Mehrgitter - Gltter
 * 
 * Glttet ein quadratisches Gitter an die Randwerte.
 * 
 * @author Thomas Klughardt
 */
public class mg_glaetter {
	// Gitterpunkte auf feinster Gitterebene (muss durch 2^n+1 teilbar sein)
	final int FINEST_GRID_LVL = 17;
	// Gitterpunkte auf grbster Gitterebene (frei whlbar)
	final int COARSEST_GRID_LVL = 3;
	final int ITERATIONEN = 100;
	
	// fr Zeitmessungen deaktivieren
	final boolean GITTER_AUSGEBEN = true;
	
	double[][] grid;
	
	boolean print = true;
	boolean FIRST_ITERATION = true;
	
	/**
	 * Erzeugt eine Instanz der Klasse, initialisiert das Gitter und fhrt die Iterationen aus.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		try {
			mg_glaetter mg = new mg_glaetter();
			mg.grid = new double[mg.FINEST_GRID_LVL][mg.FINEST_GRID_LVL];
			Random rnd = new Random();
			for (int i = 0; i < mg.FINEST_GRID_LVL; i++) {
				for (int j = 0; j < mg.FINEST_GRID_LVL; j++) {
					if (i == 0 || j == 0 || i == mg.FINEST_GRID_LVL - 1 || j == mg.FINEST_GRID_LVL - 1)
						mg.grid[i][j] = mg.getBoundaryCondition(i, j, mg.FINEST_GRID_LVL);
					else mg.grid[i][j] = 0; //rnd.nextInt(20);}
				}
			}
			
			long start = System.currentTimeMillis();
			
			for (int i = 0; i < mg.ITERATIONEN; i++) {
				mg.print = false;
	
				if (i % 10 == 0) mg.print = mg.GITTER_AUSGEBEN;
	
				mg.grid = mg.cycle(mg.FINEST_GRID_LVL, mg.FINEST_GRID_LVL, mg.grid);
				mg.FIRST_ITERATION = false;
			}
			
			long stop = System.currentTimeMillis();
			mg.print = mg.GITTER_AUSGEBEN;
			mg.printGrid(mg.FINEST_GRID_LVL, mg.FINEST_GRID_LVL, mg.grid, mg.FINEST_GRID_LVL);
			
			if (mg.GITTER_AUSGEBEN == false)
				System.out.println( "Zeit in Millisekunden: " + (int)(stop - start) + "  " + start + " -> " + stop);
		} catch (Exception e) {
			System.out.println("Ein Parameter ist schuld...");
			e.printStackTrace();
		}
	}
	
	/**
	 * Fhrt rekursiv die Mehrgitter-Glttung aus.
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Antahl Spalten
	 * @param grid - initialisiertes Gitter
	 * @return double[][] - geglttetes Gitter
	 */	
	public double[][] cycle(int rows, int cols, double[][] grid) {
		
		double[][] new_grid = null;
		
		printGrid(rows, cols, grid, rows);
		
		grid = relax(rows, cols, grid);
		
		if (rows > COARSEST_GRID_LVL && cols > COARSEST_GRID_LVL) {
			new_grid = cycle( rows/2 + 1, cols/2 + 1, restrict(rows, cols, grid) );
	 		grid = mergeGrids(rows, cols, grid, new_grid);
		}
		
		if (rows < FINEST_GRID_LVL && cols < FINEST_GRID_LVL)
			return prolong(rows, cols, grid, rows * 2 - 1); 
		
		return grid;
	}
	
	/**
	 * Fhrt eine Glttung (Relaxation) ber das Gitter aus.
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Antahl Spalten
	 * @param grid - initialisiertes Gitter
	 * @return double[][] - geglttetes Gitter
	 */
	private double[][] relax(int rows, int cols, double[][] grid) {
		for (int i = 1; i < rows - 1; i++) {
			for (int j = 1; j < cols - 1; j++) {
				grid[i][j] = (double)( (double)grid[i-1][j] + (double)grid[i][j-1] +  
							(double)grid[i+1][j] + (double)grid[i][j+1] +
							(double)grid[i][j] * 4 ) / 8;
			}
		}
		return grid;
	}
	
	/**
	 * Fhrt eine Vergrberung (Restriktion) des Gitters aus.
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Antahl Spalten
	 * @param grid - initialisiertes Gitter
	 * @return double[][] - vergrbertes Gitter
	 */
	private double[][] restrict(int rows, int cols, double[][] grid) {
		double[][] restricted_grid = new double[rows/2 + 1][cols/2 + 1];
		
		for (int i = 0; i < rows/2 + 1; i++) {
			for (int j = 0; j < cols/2 + 1; j++) {
					if (i == 0 || j == 0 || i == rows/2 || j == cols/2) {
					restricted_grid[i][j] = grid[i*2][j*2];
				} else {
					restricted_grid[i][j] = (double)( (double)grid[i*2-1][j*2-1] + (double)grid[i*2][j*2-1] + (double)grid[i*2+1][j*2-1] +
							(double)grid[i*2-1][j*2] + (double)grid[i*2][j*2] + (double)grid[i*2+1][j*2] +
							(double)grid[i*2-1][j*2+1] + (double)grid[i*2][j*2+1] + (double)grid[i*2+1][j*2+1]) / 9;
				}
			}
		}
		return restricted_grid;
	}
	
	/**
	 * Fhrt eine Verfeinerung (Prolongation) des Gitters aus.
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Antahl Spalten
	 * @param grid - initialisiertes Gitter
	 * @param nextLevel - nchst-feinerer Gitterlevel
	 * @return double[][] - vergrbertes Gitter
	 */
	private double[][] prolong(int rows, int cols, double[][] grid, int nextLevel) {
		printGrid(rows, cols, grid, rows);
		
		double[][] prolonged_grid = new double[rows*2 - 1][cols*2 - 1];
	
		for (int i = 0; i < rows*2 - 1; i+=1) {
			for (int j = 0; j < cols*2 - 1; j+=1) {
				if (i == 0 || i == rows*2 - 2 ||j == 0 || j == cols*2 - 2) {
					prolonged_grid[i][j] = getBoundaryCondition(i, j, nextLevel); //grid[i/2][j/2];
				} else {
					prolonged_grid[i][j] = (double)( (double)grid[i/2][j/2] + (double)grid[i/2][halfOf(j)] +  
							(double)grid[halfOf(i)][j/2] + (double)grid[halfOf(i)][halfOf(j)] ) / 4;
				}
			}
		}
		
		return prolonged_grid;
	}
	
	/**
	 * Liefert den Randwert in Abhngigkeit der Gitterebene und der Koordinaten.
	 * 
	 * @param i - X-Koordinate
	 * @param j - Y-Koordinate
	 * @param level - aktuelle Gitterebene
	 * @return double - zugehriger Wert
	 */
	private double getBoundaryCondition(int i, int j, int level) {
		//return Math.pow(i * (FINEST_GRID_LVL-1)/(level-1) + j * (FINEST_GRID_LVL-1)/(level-1), 2);
		//return Math.log(1 + (double)j * (FINEST_GRID_LVL-1)/(level-1) + (double)i * (FINEST_GRID_LVL-1)/(level-1));
		return Math.sin( ( (double)j * (FINEST_GRID_LVL-1)/(level-1) ) / 2) + 1;
	}
	
	/**
	 * Schreibt das bergebene Gitter.
	 * Dazu werden die Werte auf einen Maximalwert skaliert und in floats umgewandelt.
	 * Dann wird 'scale' und die Anzahl der Werte pro Zeile ausgegeben.
	 * Anschlieend werden die Gitterwerte ausgegeben.
	 * Nach dem Gitter wird noch 'draw' ausgegeben. 
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Anzahl Spalten
	 * @param grid - Gitter
	 * @param scale - Gitterebene
	 */
	public void printGrid(int rows, int cols, double[][] grid, int scale) {
		if (print == false) return;
		
		double maxValue = 0;
		double minValue = 99999999;
		System.out.println("scale "+ scale);
		
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < cols; j++) {
				if (maxValue < grid[i][j])
					maxValue = grid[i][j];
				if (minValue > grid[i][j])
					minValue = grid[i][j];
			}
		}
		
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < cols; j++) {
				System.out.print((float)( (grid[i][j] - minValue) / (maxValue-minValue) * 15000)+ "  ");
			}
			System.out.println("");
		}
		System.out.println("draw");
	}
	
	/**
	 * Fgt altes und neues Gitter nach angegebener Wichtung zusammen
	 * 
	 * @param rows - Anzahl Zeilen
	 * @param cols - Anzahl Spalten
	 * @param oldGrid - altes Gitter
	 * @param newGrid - neues Gitter
	 * @return double[][] - gemischtes Gitter
	 */
	private double[][] mergeGrids(int rows, int cols, double[][] oldGrid, double[][] newGrid) {
		if (FIRST_ITERATION) return newGrid;
		for (int i = 0; i < rows; i++)
			 for (int j = 0; j < rows; j++)
			 	oldGrid[i][j] = (oldGrid[i][j] + newGrid[i][j]) / 2; 
		
		return oldGrid;
	}
	
	/**
	 * Ermittelt die richtig gerundete Hlfte eines Integerwertes
	 *  
	 * @param n - zu halbierender Wert
	 * @return int - halbierter Wert als Integer
	 */
	private int halfOf(int n)	{
		return (int)Math.round(((double)n/2));
	}
}
