#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "SDL.h"
#include <GLUT/glut.h>

int g_w = 640;
int g_h = 480;

static SDL_Surface *gScreen;

static void initAttributes ()
{
    // Setup attributes we want for the OpenGL context
    
    int value;
    
    // Don't set color bit sizes (SDL_GL_RED_SIZE, etc)
    //    Mac OS X will always use 8-8-8-8 ARGB for 32-bit screens and
    //    5-5-5 RGB for 16-bit screens
    
    // Request a 16-bit depth buffer (without this, there is no depth buffer)
    value = 16;
    SDL_GL_SetAttribute (SDL_GL_DEPTH_SIZE, value);
    
    
    // Request double-buffered OpenGL
    //     The fact that windows are double-buffered on Mac OS X has no effect
    //     on OpenGL double buffering.
    value = 1;
    SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, value);
}

static void createSurface (int fullscreen)
{
    Uint32 flags = 0;
    
    flags = SDL_OPENGL;
    if (fullscreen)
        flags |= SDL_FULLSCREEN;
    
    // Create window
    gScreen = SDL_SetVideoMode (640, 480, 0, flags);
    if (gScreen == NULL) {
		
        fprintf (stderr, "Couldn't set 640x480 OpenGL video mode: %s\n",
                 SDL_GetError());
		SDL_Quit();
		exit(2);
	}
}

static void initGL ()
{
		static float ambient[] = {0.2, 0.2, 0.2, 1.0};
		static float diffuse[] = {1.0, 1.0, 1.0, 1.0};
		static float position[] = {0.0, 1.0, 0.0, 0.0};
		static float mat_shininess[] = {90.0};
		static float mat_specular[] = {0.8, 0.8, 0.8, 1.0};
		static float mat_diffuse[] = {0.46, 0.66, 0.795, 1.0};
		static float mat_ambient[] = {0.3, 0.4, 0.5, 1.0};
		static float lmodel_ambient[] = {0.4, 0.4, 0.4, 1.0};
		static float lmodel_localviewer[] = {0.0};
		//GLfloat map1[4] = {0.0, 0.0, 0.0, 0.0};
		//GLfloat map2[4] = {0.0, 0.0, 0.0, 0.0};
		static float fog_color[] = {0.0, 0.5, 0.9, 1.0};

		glFrontFace(GL_CCW);

		glDepthFunc(GL_LESS);
		glEnable(GL_DEPTH_TEST);

		glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
		glLightfv(GL_LIGHT0, GL_POSITION, position);
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_localviewer);
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);

		glEnable(GL_FOG);
		glFogi(GL_FOG_MODE, GL_EXP);
		glFogf(GL_FOG_DENSITY, 0.0000025);
		glFogfv(GL_FOG_COLOR, fog_color);

		glClearColor(0.0, 0.0, 0.4, 1.0);
		
		glViewport(0, 0, g_w, g_h);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60.0, (GLfloat) g_w / (GLfloat) g_h, 20000.0, 300000.0);
		glMatrixMode(GL_MODELVIEW);

}

/*
 Hier muss STDIN gelesen werden, der besprochenen Syntax folgend.
 Diese Informationen werden dann in festgelegte OpenGL draw befehle
 umgewandelt zur Laufzeit und einfach auf den Bildschirm geblittet.
 Die Stelle wo der Code eingefgt werden muss ist entsprechend
 markiert mit einem Kommentar.
*/

static void mainLoop (FILE *fp)
{
    SDL_Event event;
    int done = 0;
    int fps = 24;
	int delay = 1000/fps;
    int thenTicks = -1;
    int nowTicks;
	char cmd[32];
	float scale;
	float yf,xf,zf;
	int skip;
	int xi,yi;
	float oldZ[512];
	float angle = 0.0;
	float rotate = 0.0;
	
    while ( !feof(fp) ) {


        // Draw at 24 hz
        //     This approach is not normally recommended - it is better to
        //     use time-based animation and run as fast as possible

		// ... und zwar genau an DIESER STELLE !

/*
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glPushMatrix();
		glScalef(1, 1, 1);
			
		glTranslatef(0.0, -10000.0, -30000.0);
		glRotatef(-90,1.0,0,0);
		glRotatef(angle,0,1.0,0);
		glDisable(GL_LIGHTING);
		glEnable(GL_CULL_FACE);
*/
			scale = 1.0f;

			fscanf(fp, "%s %f\n",&cmd, &scale);
			printf("%f\n",scale);		

			glNewList(1,GL_COMPILE);
			
			glColor3f(.9,.9,.9);		
			for (yi = 0,yf = -10000.0; yf < 10000.0f; yf += 20000/scale,yi ++)
			{
				glBegin(GL_LINE_STRIP);			
				for (xi = 0,xf = -10000.0; xf < 10000.0f; xf += 20000/scale,xi ++)
				{
					fscanf(fp,"%f  ",&zf);
				//	printf("%f ",zf);

					glColor3f(zf/30000,zf/30000,zf/30000);
					glVertex3f(xf,yf,zf);
					
					if (yi)
					{
						glColor3f(oldZ[xi]/30000,oldZ[xi]/30000,oldZ[xi]/30000);
						glVertex3f(xf,yf - (20000/scale), oldZ[xi]);
												
						glColor3f(zf/30000,zf/30000,zf/30000);
						glVertex3f(xf,yf,zf);
						
					}
					
					oldZ[xi] = zf;
				}
				fscanf(fp,"\n");
			//	printf("\n");
				glEnd();			
			}
			
			glEndList();

		fscanf(fp, "%s\n",&cmd);
		memset(oldZ,0,sizeof(oldZ));

/*
		glDisable(GL_CULL_FACE);
		glPopMatrix();	
		glutSwapBuffers();
        SDL_GL_SwapBuffers ();
*/
		
		skip = 0;
		while (!skip)
		{
			rotate += 1.0;
			
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			glPushMatrix();
			glScalef(1, 1, 1);

			glRotatef(rotate,0,1.0,0);				
			glTranslatef(0.0, -10000.0, -30000.0);
		//	glRotatef(-90,1.0,0,0);
		//	glRotatef(angle,0,1.0,0);

			glDisable(GL_LIGHTING);
			glEnable(GL_CULL_FACE);

				glCallList(1);
				
			glDisable(GL_CULL_FACE);
			glPopMatrix();	
			glutSwapBuffers();
			SDL_GL_SwapBuffers ();
			
			// Time how long each draw-swap-delay cycle takes
			// and adjust delay to get closer to target framerate
			if (thenTicks > 0) {
				nowTicks = SDL_GetTicks ();
				delay += (1000/fps - (nowTicks-thenTicks));
				thenTicks = nowTicks;
				if (delay < 0)
					delay = 1000/fps;
			}
			else {
				thenTicks = SDL_GetTicks ();
			}

			SDL_Delay (delay);
			
		
			while ( SDL_PollEvent (&event) ) {
				switch (event.type) {

				case SDL_MOUSEMOTION:
					break;
				case SDL_MOUSEBUTTONDOWN:
					angle ++;
					break;
				case SDL_KEYDOWN:
					skip = 1;
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
				}
			}
		}
		
	}
}

int main(int argc, char *argv[])
{
	FILE *pf;
	
	// Init SDL video subsystem
	if ( SDL_Init (SDL_INIT_VIDEO) < 0 ) {
		
        fprintf(stderr, "Couldn't initialize SDL: %s\n",
			SDL_GetError());
		exit(1);
	}	

    // Set GL context attributes
    initAttributes ();
    
    // Create GL context
	if (argc == 2)
	{
		createSurface(1);
	}
	else
	{
		createSurface (0);
	}
        
    // Init GL state
    initGL ();
    
	pf = fopen("/storage/studium/alga/ALA/Projekt/testbild.stream","r");
//	pf = fopen("/dev/stdin","r");
	if (!pf)
	{
		printf("\n Cannot open input device.");
		return 1;
	}
	
    // Draw, get events...
    mainLoop (pf);
	fclose(pf);
	
    // Cleanup
	SDL_Quit();
	
    return 0;
}
